# Generated by Django 3.1.1 on 2021-01-28 10:46

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0003_resource_ptr_owner"),
    ]

    operations = [
        migrations.CreateModel(
            name="MetadataType",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=128, verbose_name="titre")),
            ],
        ),
        migrations.AddField(
            model_name="metadata",
            name="data_type",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="resources.metadatatype",
            ),
        ),
    ]
