# Generated by Django 3.2.2 on 2021-06-22 07:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0009_alter_metadatatype_title"),
    ]

    operations = [
        migrations.AddField(
            model_name="file",
            name="post_treated",
            field=models.BooleanField(default=True, verbose_name="traitements"),
        ),
    ]
