{% load static %}
<!DOCTYPE html>
<html lang="fr">
<head>
    <meta charset="UTF-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    <link href="{% static "ui/index.css" %}" rel="stylesheet">
    <script type="module" src="{% static "ui/index.js" %}"></script>
    <title>Jama UI - Vue 3</title>
</head>
<body>
<div id="app"></div>
</body>
</html>