# Generated by Django 3.2.2 on 2021-09-21 08:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0017_auto_20210920_1727"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="label",
            field=models.TextField(unique=True),
        ),
    ]
