# Generated by Django 5.1.5 on 2025-03-21 12:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0038_file_tiled"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="use_exiftool",
            field=models.BooleanField(default=False),
        ),
    ]
