# Generated by Django 3.1.1 on 2021-02-15 14:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0005_descendants_count"),
    ]

    operations = [
        migrations.AddField(
            model_name="collection",
            name="public_access",
            field=models.BooleanField(default=False, verbose_name="accès public"),
        ),
    ]
