# Generated by Django 5.0.1 on 2024-04-04 12:15

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0033_alter_usertask_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="usertask",
            name="project",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="resources.project",
            ),
        ),
    ]
