# Generated by Django 3.2.2 on 2021-09-22 12:25

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0019_alter_collection_project"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="admin_mail",
            field=models.EmailField(max_length=254, null=True),
        ),
    ]
