# Generated by Django 3.2.2 on 2021-09-20 14:19

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0014_auto_20210915_2024"),
    ]

    operations = [
        migrations.AddField(
            model_name="metadataset",
            name="created_at",
            field=models.DateTimeField(
                auto_now_add=True, default=django.utils.timezone.now
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="metadataset",
            name="updated_at",
            field=models.DateTimeField(auto_now=True),
        ),
    ]
