# Generated by Django 4.2.5 on 2023-10-04 12:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0028_collection_ark_resource_ark"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="ark_redirect",
            field=models.TextField(null=True),
        ),
    ]
