# Generated by Django 3.2.2 on 2021-08-30 03:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0012_auto_20210713_0807"),
    ]

    operations = [
        migrations.AddField(
            model_name="collection",
            name="is_oai_record",
            field=models.BooleanField(default=False, verbose_name="enregistrement OAI"),
        )
    ]
