# Generated by Django 3.2.2 on 2021-07-13 08:07

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0011_collection_representative"),
    ]

    operations = [
        migrations.AddField(
            model_name="collection",
            name="deleted_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="resource",
            name="deleted_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
