# Generated by Django 5.0.1 on 2024-04-04 08:34

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0032_usertask"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="usertask",
            options={"ordering": ["-updated_at"]},
        ),
    ]
