# Generated by Django 5.0.1 on 2024-04-18 12:35

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0035_remove_file_post_treated_project_use_exiftool"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="collection",
            options={
                "ordering": ("title",),
                "verbose_name": "collection",
                "verbose_name_plural": "collections",
            },
        ),
        migrations.AlterModelOptions(
            name="resource",
            options={
                "ordering": ("title",),
                "verbose_name": "ressource",
                "verbose_name_plural": "ressources",
            },
        ),
    ]
