# Generated by Django 5.0.1 on 2024-02-08 10:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0029_project_ark_redirect"),
    ]

    operations = [
        migrations.AddField(
            model_name="file",
            name="denormalized_image_height",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="hauteur de l'image"
            ),
        ),
        migrations.AddField(
            model_name="file",
            name="denormalized_image_width",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="largeur de l'image"
            ),
        ),
    ]
