# Generated by Django 3.2 on 2021-05-11 07:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0007_auto_20210225_1329"),
    ]

    operations = [
        migrations.AddField(
            model_name="metadata",
            name="expose",
            field=models.BooleanField(default=True),
        ),
    ]
