# Generated by Django 3.2.2 on 2021-11-04 16:11

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0023_auto_20211020_0948"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="metadata",
            options={
                "ordering": ["rank", "title"],
                "verbose_name": "meta-donnée",
                "verbose_name_plural": "meta-données",
            },
        ),
    ]
