{% extends 'common.html' %}
{% block title %}
    Page d'accueil
{% endblock %}
{% block content %}
    <h1>Jama - Accueil</h1>
    <p>
        Jama est un ensemble d'APIs et d'interfaces destinées à la création et à la gestion de collections de
        médias (sons, images, textes, autres...).
        Jama est développé par le <a href="https://www.certic.unicaen.fr/">CERTIC</a>.
    </p>
    <p>
        Une API <a href="https://www.jsonrpc.org/specification_v1">JSON-RPC 1.0</a> est disponible <a href="{{ BASE_PATH }}rpc/">ici</a>.
    </p>
{% endblock %}