<!DOCTYPE html>
<html lang="fr">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/water.css@2/out/water.css">
    <title>Jama - {% block title %}Page inconnue{% endblock %}</title>
</head>
<body>
{% block content %}
    <p>Ici, du contenu.</p>
{% endblock %}
</body>
</html>