# Generated by Django 3.2.2 on 2021-06-28 08:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0010_file_post_treated"),
    ]

    operations = [
        migrations.AddField(
            model_name="collection",
            name="representative",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="represents",
                to="resources.resource",
            ),
        ),
    ]
