# Generated by Django 3.2.2 on 2021-10-20 09:48

from django.db import migrations


def fixtures(*args):
    pass
    # Permission.objects.get_or_create(label="collection.public_only") # no more needed


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0022_auto_20211020_0844"),
    ]

    operations = [migrations.RunPython(fixtures)]
