# Generated by Django 5.0.1 on 2024-02-16 07:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0030_file_denormalized_image_height_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="file",
            name="text_boxes",
            field=models.JSONField(
                blank=True, null=True, verbose_name="text boxes de tesseract"
            ),
        ),
    ]
