# Generated by Django 5.0.4 on 2024-11-08 08:42

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0036_alter_collection_options_alter_resource_options"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="collection",
            name="is_oai_record",
        ),
    ]
