# Generated by Django 5.1.3 on 2024-11-28 13:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0037_remove_collection_is_oai_record"),
    ]

    operations = [
        migrations.AddField(
            model_name="file",
            name="tiled",
            field=models.BooleanField(default=False),
        ),
    ]
