# Generated by Django 4.2.5 on 2023-10-04 08:51

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0027_auto_20220131_1636"),
    ]

    operations = [
        migrations.AddField(
            model_name="collection",
            name="ark",
            field=models.CharField(max_length=64, null=True, verbose_name="nom ARK"),
        ),
        migrations.AddField(
            model_name="resource",
            name="ark",
            field=models.CharField(max_length=64, null=True, verbose_name="nom ARK"),
        ),
    ]
