# Generated by Django 3.2.2 on 2021-09-21 09:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0018_alter_project_label"),
    ]

    operations = [
        migrations.AlterField(
            model_name="collection",
            name="project",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="resources.project",
            ),
        ),
    ]
