CAN_ONLY_SHOW_PUBLIC_COLLECTIONS_CONTENT = "can only show public collections content"
CANT_CREATE_IMAGE = "Can't create image"
COLLECTION_ALREADY_EXIST_IN_PARENT = (
    "collection with same name already exists in parent collection"
)
INVALID_PROPERTY_KEY = "invalid property key"
NEED_COLLECTION_ID_OR_PROJECT_ID = "must give one of collection_id or project_id"
NEED_PROJECT_ID_IF_PARENT_ID_NULL = "project_id is needed when parent_id is null"
NEED_PROJECT_ID_OR_PARENT_ID = "use project_id OR parent_id"
NO_FOOTGUNS = "Can't shoot your own foot"
NO_METADATASET_MIX = "can't mix metadata sets when setting multiple meta values at once"
NO_PATH = "path argument is missing"
NO_PROJECT_ACCESS = "No access ({}) for project {}"
NO_PROJECT_ID = "project_id is not set"
NO_PROJECT_ROOT = "project has no root"
NO_SUCH_COLLECTION = "no such collection"
NO_SUCH_METADATA = "no such metadata"
NO_SUCH_METADATASET = "no such metadatas set"
NO_SUCH_METADATATYPE = "no such metadatatype"
NO_SUCH_PROJECT = "no such project"
NO_SUCH_PROJECT_PROPERTY = "project has no such property"
NO_SUCH_RESOURCE = "no such resource"
NO_SUCH_TAG = "no such tag"
NO_SUCH_USER = "no such user"
NOT_A_FILE = "not a file"
NOT_AN_IMAGE = "not an image"
NOTHING_TO_DO = "nothing to do"
PERM_COLLECTION_CREATE = "collection.create"
PERM_COLLECTION_DELETE = "collection.delete"
PERM_COLLECTION_READ = "collection.read"
PERM_COLLECTION_UPDATE = "collection.update"
PERM_METADATA_CREATE = "metadata.create"
PERM_METADATA_DELETE = "metadata.delete"
PERM_METADATA_READ = "metadata.read"
PERM_METADATA_UPDATE = "metadata.update"
PERM_METADATASET_CREATE = "metadataset.create"
PERM_METADATASET_DELETE = "metadataset.delete"
PERM_METADATASET_READ = "metadataset.read"
PERM_PERMISSION_UPDATE = "permission.update"
PERM_PROJECT_PROPERTY_CREATE = "project_property.create"
PERM_PROJECT_PROPERTY_DELETE = "project_property.delete"
PERM_PROJECT_PROPERTY_READ = "project_property.read"
PERM_PROJECT_PROPERTY_UPDATE = "project_property.update"
PERM_RESOURCE_DELETE = "resource.delete"
PERM_RESOURCE_READ = "resource.read"
PERM_RESOURCE_UPDATE = "resource.update"
PERM_ROLE_READ = "role.read"
PERM_TAG_DELETE = "tag.delete"
PROJECT_MISMATCH = "Can't mix items from different collections"
ROOT_COLLECTION_TITLE = "root"
SEARCH_TERMS_LIMIT = 10
SUPERUSER_NEEDED = "only for superusers"
TOO_MANY_SEARCH_TERMS = "Too many search terms."
UNKNOWN_ERROR = "unknown error"
WRONG_ARGUMENT = "wrong argument"
