# Generated by Django 3.2.2 on 2021-09-20 17:27

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0016_auto_20210920_1717"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="resource",
            name="ptr_owner",
        ),
        migrations.AlterUniqueTogether(
            name="collection",
            unique_together={("title", "project", "parent")},
        ),
        migrations.AlterUniqueTogether(
            name="file",
            unique_together={("project", "hash")},
        ),
        migrations.AlterUniqueTogether(
            name="metadata",
            unique_together={("title", "set", "project")},
        ),
        migrations.AlterUniqueTogether(
            name="metadataset",
            unique_together={("title", "project")},
        ),
        migrations.AlterUniqueTogether(
            name="tag",
            unique_together={("uid", "project")},
        ),
        migrations.RemoveField(
            model_name="collection",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="file",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="metadata",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="metadataset",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="tag",
            name="owner",
        ),
    ]
