# Generated by Django 3.2.2 on 2021-10-07 17:48

from django.db import migrations
from resources.models import FileType, FileExtension


def fixtures(*args):
    f_type, _ = FileType.objects.get_or_create(title="prout", mime="application/prout")
    f_ext, _ = FileExtension.objects.get_or_create(label="prout")
    f_type.extensions.add(f_ext)


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0020_project_admin_mail"),
    ]

    operations = [migrations.RunPython(fixtures)]
