# Generated by Django 5.0.1 on 2024-04-14 13:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0034_usertask_project"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="file",
            name="post_treated",
        ),
        migrations.AddField(
            model_name="project",
            name="use_exiftool",
            field=models.BooleanField(default=True),
        ),
    ]
