# Generated by Django 3.2.2 on 2021-05-11 11:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0008_metadata_expose"),
    ]

    operations = [
        migrations.AlterField(
            model_name="metadatatype",
            name="title",
            field=models.CharField(max_length=128, unique=True, verbose_name="titre"),
        ),
    ]
